"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateApis = void 0;
const utils_1 = require("./utils");
const strings_1 = require("./strings");
const jsdoc_1 = require("./utilities/jsdoc");
function generateApis(apis, types, config) {
    let code = strings_1.SERVICE_BEGINNING;
    try {
        const apisCode = apis
            .sort(({ serviceName }, { serviceName: _serviceName }) => utils_1.isAscending(serviceName, _serviceName))
            .reduce((prev, { contentType, summary, deprecated, serviceName, queryParamsTypeName, pathParams, requestBody, headerParams, isQueryParamsNullable, isHeaderParamsNullable, responses, method, endPoint, pathParamsRefString, additionalAxiosConfig, security, }) => {
            return (prev +
                `
${jsdoc_1.getJsdoc({
                    description: summary,
                    deprecated: deprecated ? strings_1.DEPRECATED_WARM_MESSAGE : undefined,
                })}export const ${serviceName} = (
    ${
                /** Path parameters */
                pathParams
                    .map(({ name, required, schema, description }) => utils_1.getDefineParam(name, required, schema, config, description))
                    .join(",")}${pathParams.length > 0 ? "," : ""}${
                /** Request Body */
                requestBody
                    ? `${utils_1.getDefineParam("requestBody", true, requestBody, config)},`
                    : ""}${
                /** Query parameters */
                queryParamsTypeName
                    ? `${utils_1.getParamString("queryParams", !isQueryParamsNullable, queryParamsTypeName)},`
                    : ""}${
                /** Header parameters */
                headerParams
                    ? `${utils_1.getParamString("headerParams", !isHeaderParamsNullable, headerParams)},`
                    : ""}configOverride?:AxiosRequestConfig
): Promise<SwaggerResponse<${responses ? utils_1.getTsType(responses, config) : "any"}>> => {
  ${deprecated
                    ? `
  if (__DEV__) {
    console.warn(
      "${serviceName}",
      "${strings_1.DEPRECATED_WARM_MESSAGE}",
    );
  }`
                    : ""}
  return Http.${method}Request(
    ${pathParamsRefString
                    ? `template(${serviceName}.key,${pathParamsRefString})`
                    : `${serviceName}.key`},
    ${queryParamsTypeName ? "queryParams" : "undefined"},
    ${requestBody
                    ? contentType === "multipart/form-data" ||
                        contentType === "application/x-www-form-urlencoded"
                        ? "objToForm(requestBody)"
                        : "requestBody"
                    : "undefined"},
    ${security},
    overrideConfig(${additionalAxiosConfig},
      configOverride,
    )
  )
}

/** Key is end point string without base url */
${serviceName}.key = "${endPoint}";
`);
        }, "");
        code +=
            types.reduce((prev, { name: _name }) => {
                const name = utils_1.getSchemaName(_name);
                if (!utils_1.isMatchWholeWord(apisCode, name)) {
                    return prev;
                }
                return prev + ` ${name},`;
            }, "import {") + '}  from "./types"\n';
        code += strings_1.SERVICE_NEEDED_FUNCTIONS;
        code += apisCode;
        return code;
    }
    catch (error) {
        console.error(error);
        return "";
    }
}
exports.generateApis = generateApis;
//# sourceMappingURL=generateApis.js.map