"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateTypes = void 0;
const utils_1 = require("./utils");
const strings_1 = require("./strings");
const jsdoc_1 = require("./utilities/jsdoc");
function generateTypes(types, config) {
    let code = strings_1.AUTOGENERATED_COMMENT;
    try {
        code += types
            .sort(({ name }, { name: _name }) => utils_1.isAscending(name, _name))
            .reduce((prev, { name: _name, schema, description }) => {
            const name = utils_1.getSchemaName(_name);
            prev += `
        ${jsdoc_1.getJsdoc(Object.assign(Object.assign({}, schema), { description: description || (schema === null || schema === void 0 ? void 0 : schema.description), deprecated: (schema === null || schema === void 0 ? void 0 : schema.deprecated)
                    ? (schema === null || schema === void 0 ? void 0 : schema["x-deprecatedMessage"]) || String(schema === null || schema === void 0 ? void 0 : schema.deprecated)
                    : undefined }))}
        ${getTypeDefinition(name, schema, config)}
        `;
            return prev;
        }, "");
        return code;
    }
    catch (error) {
        console.error({ error });
        return "";
    }
}
exports.generateTypes = generateTypes;
function getTypeDefinition(name, schema = {}, config) {
    const { type, enum: Enum, "x-enumNames": enumNames, allOf, oneOf, items, $ref, additionalProperties, properties, } = schema;
    if (Enum) {
        if (config.generateEnumAsType) {
            return `export type ${name} =${Enum.map((e) => `"${e}"`).join(" | ")};`;
        }
        return `export enum ${name} {${Enum.map((e, index) => `${enumNames ? enumNames[index] : e}=${typeof e === "string" ? `"${e}"` : `${e}`}`)}}`;
    }
    if (allOf || oneOf) {
        return `export type ${name} = ${utils_1.getTsType(schema, config)}`;
    }
    if (type === "array" && items) {
        return `export type ${name} = ${utils_1.getTsType(items, config)}[]`;
    }
    if ($ref) {
        return `export type ${name} = ${utils_1.getRefName($ref)}`;
    }
    if (type === "object") {
        const typeObject = utils_1.getTsType(schema, config);
        if ((additionalProperties || properties) && !oneOf) {
            return `export interface ${name} ${typeObject}`;
        }
        return `export type ${name} = ${typeObject}`;
    }
    if (type === "string") {
        return `export type ${name} = ${type}`;
    }
    return `export type ${name} = any`;
}
//# sourceMappingURL=generateTypes.js.map