"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generate = void 0;
const fs_1 = require("fs");
const prettier_1 = require("prettier");
const strings_1 = require("./strings");
const getJson_1 = require("./getJson");
const generator_1 = require("./generator");
const tsc_prog_1 = require("tsc-prog");
const utils_1 = require("./utils");
const generator_2 = require("./signalR/generator");
const swaggerToOpenApi_1 = require("./utilities/swaggerToOpenApi");
const mock_1 = require("./mock");
const chalk_1 = __importDefault(require("chalk"));
const updateJson_1 = require("./updateJson");
//@ts-ignore
const recursive_readdir_1 = __importDefault(require("recursive-readdir"));
/** @param config If isn't defined will be use swagger.config.json instead */
async function generate(config, cli) {
    config = config !== null && config !== void 0 ? config : getSwaggerConfig();
    const configs = Array.isArray(config) ? config : [config];
    configs.forEach((con) => {
        generateService(con, cli);
    });
}
exports.generate = generate;
const generateService = async (config, cli) => {
    var _a, _b, _c, _d, _e;
    config = Object.assign(Object.assign({}, config), { tag: (_a = cli === null || cli === void 0 ? void 0 : cli.tag) !== null && _a !== void 0 ? _a : config.tag, local: (_b = cli === null || cli === void 0 ? void 0 : cli.local) !== null && _b !== void 0 ? _b : config.local, branch: (_c = cli === null || cli === void 0 ? void 0 : cli.branch) !== null && _c !== void 0 ? _c : config.branch });
    const { url, hub, dir, prettierPath, language, mock, tag, keepJson, reactHooks, local, } = config;
    const isToJs = language === "javascript";
    if (!fs_1.existsSync(dir)) {
        fs_1.mkdirSync(dir);
    }
    const prettierOptions = getPrettierOptions(prettierPath);
    try {
        const swaggerJsonPath = `${dir}/swagger.json`;
        let input;
        if (local) {
            input = getLocalJson(dir);
        }
        else {
            if (!url) {
                throw new Error("Add url in swagger.config.json ");
            }
            if (typeof url === "string") {
                input = await getJson_1.getJson(url);
            }
            else {
                input = await getJson_1.getJson(await utils_1.getCurrentUrl(config));
            }
            if (input.swagger) {
                utils_1.majorVersionsCheck("2.0.0", input.swagger);
                // convert swagger v2 to openApi v3
                config._isSwagger2 = true;
                input = await swaggerToOpenApi_1.swaggerToOpenApi(input);
            }
            else {
                utils_1.majorVersionsCheck("3.0.0", input.openapi);
            }
        }
        if (keepJson) {
            try {
                if (!(tag === null || tag === void 0 ? void 0 : tag.length)) {
                    fs_1.writeFileSync(swaggerJsonPath, JSON.stringify(input));
                }
                else {
                    const oldJson = getLocalJson(dir);
                    input = updateJson_1.partialUpdateJson(oldJson, input, tag);
                    fs_1.writeFileSync(swaggerJsonPath, JSON.stringify(input));
                }
            }
            catch (error) {
                chalk_1.default.red(error);
                chalk_1.default.red("keepJson failed");
            }
        }
        const { code, hooks, type } = generator_1.generator(input, config);
        if (mock) {
            mock_1.generateMock(input, config);
        }
        fs_1.writeFileSync(`${dir}/services.ts`, code);
        console.log(chalk_1.default.yellowBright("services Completed"));
        fs_1.writeFileSync(`${dir}/types.ts`, type);
        console.log(chalk_1.default.yellowBright("types Completed"));
        if (reactHooks && hooks) {
            fs_1.writeFileSync(`${dir}/hooks.ts`, hooks);
            if (!fs_1.existsSync(`${dir}/hooksConfig.${isToJs ? "js" : "ts"}`)) {
                fs_1.writeFileSync(`${dir}/hooksConfig.ts`, strings_1.FILE_HOOKS_CONFIG);
            }
            console.log(chalk_1.default.yellowBright("hooks Completed"));
        }
        fs_1.writeFileSync(`${dir}/httpRequest.ts`, strings_1.HTTP_REQUEST);
        console.log(chalk_1.default.yellowBright("httpRequest Completed"));
        if (!fs_1.existsSync(`${dir}/config.${isToJs ? "js" : "ts"}`)) {
            fs_1.writeFileSync(`${dir}/config.ts`, strings_1.CONFIG.replace("${AUTO_REPLACE_BASE_URL}", ((_e = (_d = input.servers) === null || _d === void 0 ? void 0 : _d[0]) === null || _e === void 0 ? void 0 : _e.url) || ""));
            console.log(chalk_1.default.yellowBright("config Completed"));
        }
        // signalR hub definition
        let hubCode = null;
        if (hub) {
            const hubJson = hub ? await getJson_1.getJson(hub) : null;
            hubCode = generator_2.signalRGenerator(hubJson, config);
            hubCode && fs_1.writeFileSync(`${dir}/hub.ts`, hubCode);
            console.log(chalk_1.default.yellowBright("hub Completed"));
        }
        if (isToJs) {
            const files = [
                hubCode && "hub",
                ...(url || local
                    ? [
                        ...(reactHooks ? ["hooks", "hooksConfig"] : []),
                        "config",
                        "httpRequest",
                        "services",
                        "types",
                    ]
                    : []),
            ].filter(Boolean);
            convertTsToJs(dir, files);
        }
        recursive_readdir_1.default(dir, function (err, files) {
            if (err) {
                console.log(chalk_1.default.redBright(Error));
                return;
            }
            files.forEach((file) => {
                if (file.endsWith(".ts") || file.endsWith(".js")) {
                    formatFile(file, prettierOptions);
                }
                if (file.endsWith(".json")) {
                    formatFile(file, Object.assign(Object.assign({}, prettierOptions), { parser: "json" }));
                }
            });
        });
        console.log(chalk_1.default.greenBright("All Completed"));
    }
    catch (error) {
        console.log(chalk_1.default.redBright(error));
        console.log(chalk_1.default.redBright("failed"));
    }
};
function formatFile(filePath, prettierOptions) {
    const code = fs_1.readFileSync(filePath).toString();
    fs_1.writeFileSync(filePath, prettier_1.format(code, prettierOptions));
}
function convertTsToJs(dir, files) {
    tsc_prog_1.build({
        basePath: ".",
        compilerOptions: {
            listFiles: true,
            outDir: dir,
            declaration: true,
            skipLibCheck: true,
            module: "esnext",
            target: "esnext",
            lib: ["esnext"],
        },
        files: files.map((file) => `${dir}/${file}.ts`),
    });
    files.forEach((file) => {
        if (fs_1.existsSync(`${dir}/${file}.ts`)) {
            fs_1.rmdirSync(`${dir}/${file}.ts`, { recursive: true });
        }
    });
}
function getSwaggerConfig() {
    try {
        const config = JSON.parse(fs_1.readFileSync("swagger.config.json").toString());
        if (!config) {
            throw "";
        }
        return config;
    }
    catch (error) {
        try {
            return JSON.parse(fs_1.readFileSync("./swaggerConfig.json").toString()); // backward compatible for  v1
        }
        catch (_a) {
            throw new Error("Please define swagger.config.json");
        }
    }
}
function getPrettierOptions(prettierPath) {
    let prettierOptions = {};
    if (prettierPath && fs_1.existsSync(prettierPath)) {
        prettierOptions = JSON.parse(fs_1.readFileSync(prettierPath).toString());
    }
    else {
        if (fs_1.existsSync(".prettierrc")) {
            prettierOptions = JSON.parse(fs_1.readFileSync(".prettierrc").toString());
        }
        else if (fs_1.existsSync("prettier.json")) {
            prettierOptions = JSON.parse(fs_1.readFileSync("prettier.json").toString());
        }
    }
    if (!prettierOptions.parser) {
        prettierOptions.parser = "typescript";
    }
    return prettierOptions;
}
function getLocalJson(dir) {
    const swaggerJsonPath = `${dir}/swagger.json`;
    try {
        const old = fs_1.readFileSync(swaggerJsonPath).toString();
        return JSON.parse(old);
    }
    catch (error) {
        chalk_1.default.red("swagger.json file not found. You should set keepJson true to save json then run swag-ts without tag to save that");
        throw error;
    }
}
//# sourceMappingURL=index.js.map