"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.composeMockData = void 0;
const util_1 = require("./util");
const parse_1 = require("./parse");
const dataType_1 = require("./dataType");
const utils_1 = require("../utils");
// Compose mock data
const composeMockData = (responses, schemas) => {
    const ret = {};
    Object.keys(responses).forEach((path) => {
        const res = responses[path];
        const pathKey = util_1.normalizePath(path);
        let response = "";
        if (!res) {
            return;
        }
        Object.entries(res.response).forEach(([status, content]) => {
            const val = (content === null || content === void 0 ? void 0 : content["application/json"]) ||
                (content === null || content === void 0 ? void 0 : content["application/octet-stream"]) ||
                (content === null || content === void 0 ? void 0 : content["multipart/form-data"]);
            if (!val) {
                return;
            }
            if (val.example) {
                response = val.example;
            }
            else if (val.examples) {
                const examplesKey = Object.keys(val.examples);
                if (examplesKey.length <= 1) {
                    response = val.examples;
                }
                else {
                    // for (const [key, example] of Object.entries<any>(val.examples)) {
                    //   const extendedPathKey = pathKey + "_" + normalizeName(key);
                    //   response = example["value"];
                    // }
                }
            }
            else if ("schema" in val) {
                const { schema } = val;
                const ref = schema[parse_1.REF];
                if (ref) {
                    const schemaName = utils_1.getRefName(ref);
                    if (schemaName) {
                        response = schemas[schemaName];
                    }
                }
                else {
                    if (dataType_1.isObject(schema)) {
                        response = parse_1.parseObject(schema, schemas);
                    }
                    else if (dataType_1.isArray(schema)) {
                        response = parse_1.parseArray(schema, schemas);
                    }
                    else if (schema.properties) {
                        response = schema.properties;
                    }
                    else if (schema.type) {
                        response = dataType_1.DataType.defaultValue(schema);
                    }
                }
            }
            ret[pathKey] = {
                method: res.method,
                path: res.path,
                response: { [status]: response },
            };
        });
    });
    return ret;
};
exports.composeMockData = composeMockData;
//# sourceMappingURL=compose.js.map