"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseArray = exports.parseObject = exports.getSchemaData = exports.pickOneOf = exports.mergeAllOf = exports.REF = void 0;
const utils_1 = require("../utils");
const dataType_1 = require("./dataType");
exports.REF = "$ref";
const mergeAllOf = (properties, schemas) => {
    let ret = {};
    properties.forEach((property) => {
        if (dataType_1.isReferenceObject(property)) {
            const schemaName = utils_1.getRefName(property[exports.REF]);
            if (schemaName) {
                const schemaData = exports.getSchemaData(schemas, schemaName);
                ret = Object.assign({}, ret, schemaData);
            }
        }
        else {
            const parsed = exports.parseObject(property, schemas);
            ret = Object.assign({}, ret, parsed);
        }
    });
    return ret;
};
exports.mergeAllOf = mergeAllOf;
const pickOneOf = (properties, schemas) => {
    const property = properties[0];
    if (dataType_1.isReferenceObject(property)) {
        const schemaName = utils_1.getRefName(property[exports.REF]);
        if (schemaName) {
            const schemaData = exports.getSchemaData(schemas, schemaName);
            return schemaData;
        }
    }
    const parsed = exports.parseObject(property, schemas);
    return Object.assign({}, parsed);
};
exports.pickOneOf = pickOneOf;
// Retrieve mock data of schema.
const getSchemaData = (schemas, name) => {
    const schema = schemas[name];
    if (dataType_1.isReferenceObject(schema)) {
        const schemaName = utils_1.getRefName(schema[exports.REF]);
        return schemaName ? exports.getSchemaData(schemas, schemaName) : {};
    }
    if (dataType_1.isAllOf(schema)) {
        return exports.mergeAllOf(schema.allOf, schemas);
    }
    else if (dataType_1.isArray(schema)) {
        return exports.parseArray(schema, schemas);
    }
    else if (dataType_1.isObject(schema)) {
        return exports.parseObject(schema, schemas);
    }
    else if (schema.type) {
        return dataType_1.DataType.defaultValue(schema);
    }
    return schema;
};
exports.getSchemaData = getSchemaData;
const parseObject = (obj, schemas) => {
    if (obj.example)
        return obj.example;
    if (!obj.properties) {
        return {};
    }
    return Object.keys(obj.properties).reduce((acc, key) => {
        const property = obj.properties[key];
        if (dataType_1.isReferenceObject(property)) {
            const schemaName = utils_1.getRefName(property[exports.REF]);
            if (schemaName) {
                const schema = exports.getSchemaData(schemas, schemaName);
                acc[key] = schema;
            }
            return acc;
        }
        if (dataType_1.isAllOf(property)) {
            acc[key] = exports.mergeAllOf(property.allOf, schemas);
        }
        else if (dataType_1.isOneOf(property)) {
            acc[key] = exports.pickOneOf(property.oneOf, schemas);
        }
        else if (dataType_1.isAnyOf(property)) {
            acc[key] = exports.pickOneOf(property.anyOf, schemas);
        }
        else if (dataType_1.isObject(property)) {
            acc[key] = exports.parseObject(property, schemas);
        }
        else if (dataType_1.isArray(property)) {
            acc[key] = exports.parseArray(property, schemas);
        }
        else if (property.type) {
            acc[key] = dataType_1.DataType.defaultValue(property);
        }
        return acc;
    }, {});
};
exports.parseObject = parseObject;
const parseArray = (arr, schemas) => {
    if (dataType_1.isReferenceObject(arr.items)) {
        const schemaName = utils_1.getRefName(arr.items[exports.REF]);
        if (schemaName) {
            const schema = exports.getSchemaData(schemas, schemaName);
            return [schema];
        }
        return [];
    }
    else if (arr.example) {
        return arr.example;
    }
    else if (arr.items.type) {
        return [exports.parseObject(arr.items, schemas)];
    }
    return [];
};
exports.parseArray = parseArray;
//# sourceMappingURL=parse.js.map